use palette::cam16::Cam16Jch;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct HctDiffference {
    pub hue: Differ,
    pub chroma: Differ,
    pub lightness: Differ,
}

impl ColorDifference for Cam16Jch<f32> {
    type Difference = HctDiffference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let hue = other.hue.into_positive_degrees() - self.hue.into_positive_degrees();
        let chroma = other.chroma - self.chroma;
        let lightness = other.lightness - self.lightness;

        HctDiffference {
            hue: Differ {
                delta: hue,
                percent: hue / self.hue.into_positive_degrees(),
            },
            chroma: Differ {
                delta: chroma,
                percent: chroma / self.chroma,
            },
            lightness: Differ {
                delta: lightness,
                percent: lightness / self.lightness,
            },
        }
    }

    fn difference_relative(&self, other: &Self) -> Self::Difference {
        let hue = if self.hue.into_positive_degrees() == 0.0 {
            0.0
        } else {
            (other.hue.into_positive_degrees() - self.hue.into_positive_degrees())
                / (360.0 - self.hue.into_positive_degrees())
        };
        let chroma = if self.chroma == 0.0 {
            0.0
        } else {
            (other.chroma - self.chroma) / self.chroma
        };
        let lightness = if self.lightness == 0.0 {
            0.0
        } else {
            (other.lightness - self.lightness) / (100.0 - self.lightness)
        };

        HctDiffference {
            hue: Differ {
                delta: other.hue.into_positive_degrees() - self.hue.into_positive_degrees(),
                percent: hue,
            },
            chroma: Differ {
                delta: other.chroma - self.chroma,
                percent: chroma,
            },
            lightness: Differ {
                delta: other.lightness - self.lightness,
                percent: lightness,
            },
        }
    }
}
